# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## OUTBREAK PROBABILITY ========================================================

# Script authors:
#     Chloe Fletcher        ORCID: 0000-0002-6705-7605
#     Martín Lotto Batista  ORCID: 0000-0002-9437-5270
#     Giovenale Moirano     ORCID: 0000-0001-8748-3321
#     Prof. Rachel Lowe     ORCID: 0000-0003-3939-7343

# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

outbreak_probability <- function(data, 
                                 response = NULL,
                                 samples = NULL, 
                                 lag = NA,
                                 epi_floor = NULL,
                                 nyears = NULL,
                                 nmonths = 12,
                                 q = NULL,
                                 start_id = NULL) {
  
  # create empty output object
  out <- NULL
  
  # calculate 75th percentile for each month in time series
  # (note, this is only informed by years preceding the prediction month)
  for (j in sort(unique(data[["year_index"]]))){
    add <- data %>%
      select(year_index, month, all_of(response)) %>%
      filter(year_index < j) %>% 
      group_by(month) %>% 
      summarise(thresh = quantile(!!sym(response), q), .groups = "drop") %>% 
      mutate(thresh = ifelse(thresh < epi_floor, epi_floor, thresh),
             year_index = j)
    
    out <- bind_rows(out, add)
  }
  
  # cut down to relevant years
  out <- out[out$year_index >= data[start_id, "year_index"], ]
  data.cut <- data[start_id:nrow(data), ]
  
  # compute outbreak probability
  out <- out %>% 
      bind_cols(list(cases = data.cut[[response]],
                     as.data.frame(samples))) %>%
      rowwise() %>% 
      mutate(pred_mean = mean(c_across(paste0("V", c(1:ncol(samples))))),
             pred_lower = quantile(c_across(paste0("V", c(1:ncol(samples)))),
                                   0.025, na.rm=TRUE),
             pred_upper = quantile(c_across(paste0("V", c(1:ncol(samples)))),
                                   0.975, na.rm=TRUE),
             across(paste0("V", c(1:ncol(samples))), 
                    ~ifelse(.x >= thresh, 1, 0)), 
             obs_outbreak=ifelse(cases>thresh, 1, 0),
             pred_outbreak_prob = sum(c_across(paste0("V", c(1:ncol(samples))))) / 
               ncol(samples)) %>%
      dplyr::select(year_index, month, cases, thresh, pred_mean, 
                    pred_lower, pred_upper, obs_outbreak, pred_outbreak_prob) %>% 
      ungroup()
  
  return(out)
}


## END