# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## PREDICTIVE ABILITY ==========================================================

# Script authors:
#     Chloe Fletcher        ORCID: 0000-0002-6705-7605
#     Martín Lotto Batista  ORCID: 0000-0002-9437-5270
#     Giovenale Moirano     ORCID: 0000-0001-8748-3321
#     Prof. Rachel Lowe     ORCID: 0000-0003-3939-7343

# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

predictive_ability <- function(outbreak_data = NULL, 
                               samples = NULL, 
                               label = NULL,
                               response = NULL) {
  
  # calculate CRPS
  crps <- median(scoringutils::crps_sample(outbreak_data[[response]], samples))
  
  # create ROC curve
  ROC <- pROC::roc(outbreak_data[["obs_outbreak"]], # Binary observed outbreaks
                   outbreak_data[["pred_outbreak_prob"]], # Outbreak probability
                   direction = "<", ci = TRUE, quiet = TRUE)
  
  # find outbreak trigger threshold
  trigger <- pROC::coords(ROC, x = "best", best.method = "closest.topleft")
  
  # compute true positive rate and false positive rate
  t <- outbreak_data %>% 
    mutate(pred_outbreak = ifelse(pred_outbreak_prob > trigger[["threshold"]],
                                  1, 0))
  
  mat <- caret::confusionMatrix(data = factor(t[["pred_outbreak"]]),
                                reference = factor(t[["obs_outbreak"]]),
                                positive = "1")[["byClass"]]
  
  # Save results
  table.out <- data.frame(model = label,
                          crps = round(crps, 3),
                          auc = round(ROC[["auc"]][[1]], 2),
                          auc.lower = round(ROC[["ci"]][[1]], 2),
                          auc.upper = round(ROC[["ci"]][[3]], 2),
                          trigger = round(trigger[["threshold"]], 2),
                          tpr = round(mat[["Sensitivity"]], 2),
                          fpr = round(1 - mat[["Specificity"]], 2))
  
  return(table.out)
}


## END